precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;//portrait mask
uniform sampler2D inputImageTexture2;//front effect image
uniform sampler2D inputImageTexture3;//back effect image
uniform int blendMode;//0,screen;1,normal;2,multiply;3 unpremultiply+normal
uniform int removeFrontEffect;

vec4 unpremultiply(vec4 s) {
    return vec4(s.rgb/max(s.a,0.00001), s.a);
}

vec4 premultiply(vec4 s) {
    return vec4(s.rgb * s.a, s.a);
}

vec4 normalBlend(vec4 Cb, vec4 Cs) {
    vec4 dst = premultiply(Cb);
    vec4 src = premultiply(Cs);
    return unpremultiply(src + dst * (1.0 - src.a));
}
vec4 normalBlendNoPremultiply(vec4 Cb, vec4 Cs) {
    vec4 dst = premultiply(Cb);
    vec4 src = (Cs);
    return unpremultiply(src + dst * (1.0 - src.a));
}

vec3 saturate(vec3 rgb) {
    return clamp(rgb, vec3(0.0), vec3(1.0));
}

vec4 saturate2(lowp vec4 v) {
    return clamp(v, 0.0, 1.0);
}

vec4 blendBaseAlpha(vec4 Cb, vec4 Cs, vec4 B) {
    vec4 Cr = vec4((1.0 - Cb.a) * Cs.rgb + Cb.a * saturate(B.rgb), Cs.a);
    return normalBlend(Cb, Cr);
}

// multiply
vec4 multiplyBlend(vec4 Cb, vec4 Cs) {
    vec4 B = saturate2(vec4(Cb.rgb * Cs.rgb, Cs.a));
    return blendBaseAlpha(Cb, Cs, B);
}

vec4 screenBlend(vec4 Cb, vec4 Cs) {
    vec4 White = vec4(1.0);
    vec4 B = White - ((White - Cs) * (White - Cb));
    return blendBaseAlpha(Cb, Cs, B);
}

float lum(vec4 color) {
    return  (0.2126 * color.r) + (0.7152 * color.g) + (0.0722 * color.b);
}

void main()
{
    lowp vec2 uv  = textureCoordinate;
    uv.y = 1.0 - uv.y;//left,top is (0,0)

    vec4 srcColor = texture2D(inputImageTexture, textureCoordinate);

    vec4 portraitColor     = texture2D(inputImageTexture1, textureCoordinate);
    vec4 frontEffectColor  = texture2D(inputImageTexture2, uv);
    vec4 backEffectColor   = texture2D(inputImageTexture3, uv);

    vec4 outCol = srcColor;

    if(blendMode == 0) {
        //backEffectColor.a = getColorGray(backEffectColor);
        //frontEffectColor.a = getColorGray(frontEffectColor);
        vec4 color = screenBlend(backEffectColor, srcColor);
        vec4 color1 = normalBlend(color, portraitColor);

        if(removeFrontEffect > 0) {
            outCol = color1;
        } else {
            outCol = screenBlend(frontEffectColor, color1);
        }

        //        outCol.a = srcColor.a;
        //        if(outCol.a < 0.00001) {
        //            float brightness = lum(frontEffectColor);
        //            if(brightness > 0.0) outCol.a = brightness;
        //            brightness = lum(backEffectColor);
        //            if(brightness > 0.0) outCol.a = brightness;
        //        }
    } else if(blendMode == 1) {
        vec4 color = normalBlendNoPremultiply(srcColor, backEffectColor);
        vec4 color1 = normalBlend(color, portraitColor);

        if(removeFrontEffect > 0) {
            outCol = color1;
        } else {
            outCol = normalBlendNoPremultiply(color1, frontEffectColor);
        }

    } else if(blendMode == 2) {
        vec4 color = multiplyBlend(srcColor, backEffectColor);
        vec4 color1 = normalBlend(color, portraitColor);

        if(removeFrontEffect > 0) {
            outCol = color1;
        } else {
            outCol = multiplyBlend(color1, frontEffectColor);
        }
    } else if(blendMode == 3) {
        vec4 color = normalBlend(srcColor, backEffectColor);
        vec4 color1 = normalBlend(color, portraitColor);

        if(removeFrontEffect > 0) {
            outCol = color1;
        } else {
            outCol = normalBlend(color1, frontEffectColor);
        }
    }

    gl_FragColor = outCol;
}