precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float effectValue;

vec2 hash22(vec2 p)
{
    vec3 p3 = fract(vec3(p.xyx) * 443.8975);
    p3 += dot(p3, p3.yzx+19.19);
    return fract(vec2((p3.x + p3.y)*p3.z, (p3.x+p3.z)*p3.y));
}
// https://registry.khronos.org/OpenGL-Refpages/gl4/html/bitCount.xhtml
// Use GLSL bitCount if available
float bitCount(float x) {
    float count = 0.0;
    for (float i = 0.0; i < 24.0; ++i) {
        float xx = floor(x);
        xx /= pow(2.0, i);
        count += mod(xx, 2.0);
    }
    return count;
}

vec2 getPos(vec2 uv, vec2 scale,float seed) {
    vec2 hash = hash22(hash22(uv+ vec2(seed, seed)));
    vec2 ihash = vec2(hash * 16777215.0);
    vec2 offset = scale * (float(bitCount(ihash.x) + bitCount(ihash.y)) - 32.0)  / 24.0;
    return uv + normalize(hash) * offset ;
}

void main()
{
    vec2 uv = textureCoordinate;
    vec2 scale = 1.0 /vec2(1080.0);
    float glassWidth = 88.0 *effectValue;
    scale *= glassWidth ;
    vec4 col = vec4(0.0,0.0,0.0,0.0);
    float step =  floor(5.0);
    for (float i = 0.0; i < step; ++i){
        col += texture2D(inputImageTexture,  getPos(uv, scale,i));
    }
    vec4 outCol = col / float(step);
    float gray = 0.299*outCol.r + 0.587*outCol.g + 0.114*outCol.b;
    outCol.rgb += 0.2 * (1.0 - gray);
    gl_FragColor = outCol;
}