precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform vec3 color;
uniform int mode;//0,normal;1,light;2,replace

vec4 unpremultiply(vec4 s) {
    return vec4(s.rgb/max(s.a,0.00001), s.a);
}

vec4 premultiply(vec4 s) {
    return vec4(s.rgb * s.a, s.a);
}

vec4 normalBlend(vec4 Cb, vec4 Cs) {
    vec4 dst = premultiply(Cb);
    vec4 src = premultiply(Cs);
    return unpremultiply(src + dst * (1.0 - src.a));
}

vec3 saturate(vec3 rgb) {
    return clamp(rgb, vec3(0.0), vec3(1.0));
}

vec4 saturate2(lowp vec4 v) {
    return clamp(v, 0.0, 1.0);
}

vec4 blendBaseAlpha(vec4 Cb, vec4 Cs, vec4 B) {
    vec4 Cr = vec4((1.0 - Cb.a) * Cs.rgb + Cb.a * saturate(B.rgb), Cs.a);
    return normalBlend(Cb, Cr);
}

float lum(vec4 color) {
    return dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
}

vec4 clipColor(vec4 C) {
    float l = lum(C);
    float n = min(C.r, min(C.g, C.b));
    float x = max(C.r, max(C.g, C.b));
    if (n < 0.0) {
        return vec4((l + ((C.rgb - l) * l) / (l - n)), C.a);
    }
    if (x > 1.0) {
        return vec4(l + (((C.rgb - l) * (1.0 - l)) / (x - l)), C.a);
    }
    return C;
}

vec4 setLum(vec4 C, float l) {
    float d = l - lum(C);
    return clipColor(vec4(C.rgb + d, C.a ));
}

vec4 colorBlend(vec4 Cb, vec4 Cs) {
    vec4 B = setLum(Cs, lum(Cb));
    return blendBaseAlpha(Cb, Cs, B);
}

void main()
{
    vec2 uv = textureCoordinate;
    vec4 outCol = texture2D(inputImageTexture, uv);
    if(mode == 1) {
        outCol.rgb *= color;
    } else if(mode == 2) {
        outCol.rgb = color;
    } else {
        outCol = colorBlend(outCol, vec4(color, 1.0));
    }

    gl_FragColor = outCol;
}