precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform float effectValue;
uniform float iTime;
uniform vec2 inputSize;

#define SlideValue(a, b, x)  ((a) + ((x) * ((b) - (a))))
#define sat(t)  clamp((t), 0.0, 1.0)
//#define mytrunc(x, num_levels) floor((x) * num_levels) / (num_levels)
#define mod(x,y) ((x) - (y) * floor((x)/(y)))

float mytrunc(float x, float num_levels) {
    return floor((x) * num_levels) / (num_levels);
}

vec2 mytrunc2(vec2 x, float num_levels) {
    return floor((x) * num_levels) / (num_levels);
}
//remaps inteval [a;b] to [0;1]
float remap(float t, float a, float b) {
    return sat((t - a) / (b - a));
}

//note: t=[0;0.5;1], y=[0;1;0]
float linterp(float t) {
    return sat(1.0 - abs(2.0 * t - 1.0));
}

vec3 spectrum_offset(float t) {
    float t0 = 3.0 * t - 1.5;
    return clamp(vec3(-t0, 1.0 - abs(t0), t0), 0.0, 1.0);
}

//note: [0;1]
float rand(vec2 n) {
    return fract(sin(dot(n.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

//note: [-1;1]
float srand(vec2 n) {
    return rand(n) * 2.0 - 1.0;
}

void main()
{
    vec2 uv = textureCoordinate;
    
    float GLITCH = 0.1;
    float swing = 6.0;

    //  if (level == 0) {
    //    GLITCH = 0.1;
    //  } else if (level == 1) {
    //    GLITCH = 0.2;
    //  } else {
    //    GLITCH = 0.2;
    //    swing = 16.0;
    //    yStep = 10.0;
    //  }

    GLITCH = SlideValue(0.05, 0.2, effectValue);
    //    swing = 16.0 * effectValue;
    //    yStep = 10.0 * effectValue;

    vec2 iResolution = inputSize;

    float aspect = iResolution.x / iResolution.y;
    //uv.y = 1.0 - uv.y;

    float time = mod(iTime, 32.0); // + modelmat[0].x + modelmat[0].z;

    //float rdist = length( (uv - vec2(0.5,0.5))*vec2(aspect, 1.0) )/1.4;
    //GLITCH *= rdist;

    float xStep = 10.0;
    float yStep = 10.0 / aspect;

    float gnm = sat(GLITCH);
    float rnd0 = rand(mytrunc2(vec2(time, time), swing));
    float r0 = sat((1.0 - gnm) * 0.7 + rnd0);

    float rnd3 = rand(vec2(mytrunc(uv.y, 10.0 * r0), time));
    float r3 = (1.0 - sat(rnd3 + 0.8)) - 0.1;

    uv.y += 0.1 * r3 * GLITCH;

    const int NUM_SAMPLES = 10;
    const float RCP_NUM_SAMPLES_F = 1.0 / float(NUM_SAMPLES);

    vec4 sum = vec4(0.0);
    vec3 wsum = vec3(0.0);
    for (int i = 0; i < NUM_SAMPLES; ++i) {
        float t = float(i) * RCP_NUM_SAMPLES_F;
        uv.x = sat(uv.x + 0.01 * t);
        vec4 samplecol = texture2D(inputImageTexture, uv);
        vec3 s = spectrum_offset(t);
        samplecol.rgb = samplecol.rgb * s;
        sum += samplecol;
        wsum += s;
    }
    sum.rgb /= wsum;
    sum.a *= RCP_NUM_SAMPLES_F;

    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    color.rgb = sum.rgb;

    gl_FragColor = color;
}